--
-- ObjectEditor
--
--
-- @author jodamm
-- @date 17/04/2017

print("----ObjectEditor");
ObjectEditor = {};

registerHandTool("hedgeTrimmer", hedgeTrimmer);

ObjectEditor.XmlId = nil;
ObjectEditor.XmlFileName = "";
ObjectEditor.nonCollisionObjects = {};
ObjectEditor.numObjectsToRemove = 0;
    
function ObjectEditor:loadMap(savegame)
    if g_currentMission.missionInfo.savegameDirectory ~= nil then
        ObjectEditor.XmlFileName = g_currentMission.missionInfo.savegameDirectory.."/ObjectEditor.xml";
        if not fileExists(ObjectEditor.XmlFileName) then            
            ObjectEditor.XmlId = createXMLFile("ObjectEditorXML", g_currentMission.missionInfo.savegameDirectory.."/ObjectEditor.xml", "ObjectEditor");
            setXMLString(ObjectEditor.XmlId, "ObjectEditor.ObjectsToRemove.Object(0)#IndexPath" , "4>0|5|6");
            setXMLString(ObjectEditor.XmlId, "ObjectEditor.ObjectsToRemove.Object(0)#Comment" , "First entry will not be removed from the map, it is a template for you");
            setXMLString(ObjectEditor.XmlId, "ObjectEditor.ObjectsToMove.Object(0)#IndexPath" , "4>1|4|10|45");
            setXMLString(ObjectEditor.XmlId, "ObjectEditor.ObjectsToMove.Object(0)#Position" , "-10 0.5 0");
            setXMLString(ObjectEditor.XmlId, "ObjectEditor.ObjectsToMove.Object(0)#Rotation" , "90 0 0");                       
            setXMLString(ObjectEditor.XmlId, "ObjectEditor.ObjectsToMove.Object(0)#Comment" , "First entry will not be moved on the map, it is a template for you.");
            ObjectEditor.searchNonCollisonObjects();
            local i = 0;
            for key, paths in pairs(ObjectEditor.nonCollisionObjects) do
                local xmlPath = string.format("ObjectEditor.ObjectsWithoutCollision.Object(%d)#", i);
                setXMLString(ObjectEditor.XmlId, xmlPath.."key" , key);
                setXMLString(ObjectEditor.XmlId, xmlPath.."paths" , paths);
                i = i + 1
            end                        
            saveXMLFile(ObjectEditor.XmlId);
--            DebugUtil.printTableRecursively(ObjectEditor.nonCollisionObjects,"-",0,2);
        else
            ObjectEditor.XmlId = loadXMLFile("ObjectEditorXML", ObjectEditor.XmlFileName);
            if ObjectEditor.XmlId ~= nil then
                local i = 0;
                local xmlPath = string.format("ObjectEditor.ObjectsWithoutCollision.Object(%d)#", i)
                while hasXMLProperty(ObjectEditor.XmlId, xmlPath.."key") do                    
                    local xz = getXMLString(ObjectEditor.XmlId, xmlPath.."key");
                    local paths = getXMLString(ObjectEditor.XmlId, xmlPath.."paths");
                    if xz ~= nil then
                        ObjectEditor.nonCollisionObjects[xz] = paths;
                    end
                    i = i + 1;
                    xmlPath = string.format("ObjectEditor.ObjectsWithoutCollision.Object(%d)#", i)
                end
            end
        end
    end
--    print("Table content");
--    DebugUtil.printTableRecursively(ObjectEditor.nonCollisionObjects,"-",0,2);
    if ObjectEditor.XmlId ~= nil then
        local mapNode = getChild(getRootNode(), "untitled");
        local i = 1;
        while hasXMLProperty(ObjectEditor.XmlId, string.format("ObjectEditor.ObjectsToRemove.Object(%d)#IndexPath",i)) do
            local removeNode = ObjectEditor.indexToObject(mapNode, getXMLString(ObjectEditor.XmlId, string.format("ObjectEditor.ObjectsToRemove.Object(%d)#IndexPath",i)));
            ObjectEditor.removeCollision (removeNode);
            setVisibility(removeNode, false);
            i = i + 1;
        end
        ObjectEditor.numObjectsToRemove = i;
        i = 1;
        while hasXMLProperty(ObjectEditor.XmlId, string.format("ObjectEditor.ObjectsToMove.Object(%d)#IndexPath",i)) do
            local moveNode = ObjectEditor.indexToObject(mapNode, getXMLString(ObjectEditor.XmlId, string.format("ObjectEditor.ObjectsToMove.Object(%d)#IndexPath",i)));
            local mx, my, mz = Utils.getVectorFromString(getXMLString(ObjectEditor.XmlId, string.format("ObjectEditor.ObjectsToMove.Object(%d)#Position",i)))
            local rot = Utils.getRadiansFromString(getXMLString(ObjectEditor.XmlId, string.format("ObjectEditor.ObjectsToMove.Object(%d)#Rotation",i)), 3)
            local x, y, z = getTranslation(moveNode);
            setTranslation(moveNode, x+mx, y+my, z+mz);
            x, y, z = getRotation(moveNode);
            setRotation(moveNode, x+rot[1], y+rot[2], z+rot[3]);            
            i = i + 1;
        end     
    end
    ObjectEditor.mapNode = getChild(getRootNode(), "untitled");
end

function ObjectEditor:deleteMap()
end

function ObjectEditor:mouseEvent(posX, posY, isDown, isUp, button)
end

function ObjectEditor:keyEvent(unicode, sym, modifier, isDown)
end

function ObjectEditor:update(dt)
end

function ObjectEditor:draw()
end

function ObjectEditor.saveSavegame(self, missionInfo)
    if ObjectEditor.XmlId ~= nil then
        saveXMLFile(ObjectEditor.XmlId);
    end
end

function ObjectEditor.indexToObject(node, index)
    local rootNode, childNodes = string.match(index, '(%d+)>(.*)')
--    print ("rootNode = ".. rootNode);
--    print ("childNodes = ".. childNodes);
    local nextNode = getChildAt(node, tonumber(rootNode));
--    print ("nextNode = "..nextNode);
    for nodeSub in string.gmatch(childNodes, '([^|]+)') do
        nextNode = getChildAt(nextNode, tonumber(nodeSub));        
--        print("nodeSub = "..nodeSub);
--        print("nextNode = "..nextNode);
    end
    return nextNode
end

function ObjectEditor.removeCollision(node)
    local numChilds = getNumOfChildren(node);
--        print("noclip = "..node);
        setRigidBodyType(node, "norigidbody");
    for i=0, numChilds-1 do
        childId = getChildAt(node, i);
        if getNumOfChildren(childId) > 0 then
            ObjectEditor.removeCollision(childId);
--            print("noclip = "..childId);
            setRigidBodyType(childId, "norigidbody");
        end
--        print("noclip = "..childId);
        setRigidBodyType(childId, "norigidbody");
    end     
end

function ObjectEditor.searchNonCollisonObjects()
    print("Searching....")
    local mapNode = getChild(getRootNode(), "untitled");
    local numChilds = getNumOfChildren(mapNode);
    for i=0, numChilds-1 do
        local path = i..">";
        childId = getChildAt(mapNode, i);    
        ObjectEditor.findChildsWithoutCollison(childId, path);
    end
end

function ObjectEditor.findChildsWithoutCollison(node, path)
    local numChilds = getNumOfChildren(node);
    for i=0, numChilds-1 do
        local lpath = path ..i;
        childId = getChildAt(node, i);
        local transformgroupWithTranslation = false;
        if getHasClassId(childId, ClassIds.TRANSFORM_GROUP) then
            local x, y, z = getTranslation(childId);
            if math.abs(x) > 0.001 or math.abs(y) > 0.001 or math.abs(z) > 0.001 then
                transformgroupWithTranslation = true;
            end
        end
        if not getHasClassId(childId, ClassIds.SHAPE) or (getRigidBodyType(childId) == "NoRigidBody" and not getHasClassId(childId, ClassIds.LIGHT_SOURCE)) then
            if getNumOfChildren(childId) > 0 then
                if not transformgroupWithTranslation then
                    lpath = lpath .."|";
                    ObjectEditor.findChildsWithoutCollison(childId, lpath)
                end
            else
                if getHasClassId(childId, ClassIds.SHAPE) then
                    local x, y, z = getTranslation(childId);
                    if math.abs(x) > 0.001 or math.abs(y) > 0.001 or math.abs(z) > 0.001 then
                        local xz = "x"..(math.floor(x * 0.1 + 0.5) / 0.1).."z"..(math.floor(z * 0.1 + 0.5) / 0.1);
                        if ObjectEditor.nonCollisionObjects[xz] == nil then
                            ObjectEditor.nonCollisionObjects[xz] = lpath;                        
                        else
                            ObjectEditor.nonCollisionObjects[xz] = ObjectEditor.nonCollisionObjects[xz] .. " ".. lpath;                        
                        end 
                    end
                end
            end
        end
    end
end

function ObjectEditor.getNearestNonCollisionObjects(x, y, z)
    local nearObjects = "";
    local nearestObject = 0;
    local nearestDist = 1;
    local nearestPath = "";
    for i = -10, 10, 10 do
        for j = -10, 10, 10 do
            objects = ObjectEditor.getNonCollisionObjects(x +i , z+ j)
            if objects ~= nil then
                nearObjects = nearObjects .. objects .." ";       
            end
        end
    end
--    print("Near Objects = "..nearObjects);
    if nearObjects ~= "" then
        for path in string.gmatch(nearObjects, '([^ ]+)') do
            local objectId = ObjectEditor.indexToObject(ObjectEditor.mapNode, path);
            if getVisibility(objectId) then
                local xo, yo, zo = getTranslation(objectId);
                local dist = Utils.vector2Length(x-xo, z-zo);
                if dist < nearestDist then
                    nearestObject = objectId;
                    nearestDist = dist;
                    nearestPath = path;
                end
            end        
        end
    end
    return nearestObject, nearestPath; 
end

function ObjectEditor.getNonCollisionObjects(x, z)
    local xz = "x"..(math.floor(x * 0.1 + 0.5) / 0.1).."z"..(math.floor(z * 0.1 + 0.5) / 0.1);
    return ObjectEditor.nonCollisionObjects[xz];
end

function ObjectEditor.removeObject(path)
    local removeNode = ObjectEditor.indexToObject(ObjectEditor.mapNode, path);
--    setMaterial(removeNode, Placeable.GLOW_MATERIAL, 0)
--    setShaderParameter(removeNode, "colorScale", 0, 1, 0, 1, false);
    if getVisibility(removeNode) then
        ObjectEditor.removeCollision (removeNode);
        setVisibility(removeNode, false);
        local xmlPath = string.format("ObjectEditor.ObjectsToRemove.Object(%d)#",ObjectEditor.numObjectsToRemove); 
        setXMLString(ObjectEditor.XmlId, xmlPath.."IndexPath" , path);
        setXMLString(ObjectEditor.XmlId, xmlPath.."Name" , getName(removeNode));
        ObjectEditor.numObjectsToRemove = ObjectEditor.numObjectsToRemove + 1
    end
end

g_careerScreen.saveSavegame = Utils.appendedFunction(g_careerScreen.saveSavegame, ObjectEditor.saveSavegame);

addModEventListener(ObjectEditor);

print("----ObjectEditor loaded.");