--
-- hedgeTrimmer
--
--
-- @author jodamm
-- @date 19/04/2017
--
--print("-----Hedgetrimmer");
hedgeTrimmer = {};

function hedgeTrimmer:new(isServer, isClient, customMt)
--    print("----New hedge trimmer")
    local mt = customMt
    if mt == nil then
        mt = Class(hedgeTrimmer);
    end
    local self = HandTool:new(isServer, isClient, mt)
return self
end

function hedgeTrimmer:load(xmlFilename, player)
--    print("----xmlFilename "..xmlFilename);
    if not HandTool.load(self, xmlFilename, player) then
        return false
    end
    
    local xmlFile = loadXMLFile("TempXML", xmlFilename)
    
    if self.isClient then
        self.handNode = Utils.getNoNil(Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "handTool.hedgeTrimmer.handNode#index")), self.rootNode)
        self.handNodeRotation = Utils.getRadiansFromString(Utils.getNoNil(getXMLString(xmlFile, "handTool.hedgeTrimmer.handNode#rotation"), "0 0 0"), 3)
        self.bladeNode = Utils.getNoNil(Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "handTool.hedgeTrimmer.bladeNode#index")), self.rootNode)    
    end
    self.currentHandNode = nil
    self.vibeDt = 0;
    self.hasNearestObject = false;
    self.nearestObject = nil;
    return true;
end

function hedgeTrimmer:getSaveAttributesAndNodes(nodeIdent)
end

function hedgeTrimmer:delete()
end

function hedgeTrimmer:update(dt, allowInput)
    local lockPlayerInput = false; 
    self.player.walkingIsLocked = false;
    if allowInput then        
        if InputBinding.isPressed(InputBinding.ACTIVATE_HANDTOOL) then
            if not self.hasNearestObject then
                self.hasNearestObject = true;
                local x,y,z = getWorldTranslation(self.bladeNode)
                self.nearestObject, self.nearestPath = ObjectEditor.getNearestNonCollisionObjects(x, y, z);
            end
            if self.nearestObject ~= 0 then
                  lockPlayerInput = true;
                  self.player.walkingIsLocked = true
                  self.cutTime = self.cutTime + dt;
--                print(nearObjects);
--                ObjectEditor.removeObject(nearObjects);
                  self.vibeDt = self.vibeDt + dt; 
                  if self.vibeDt > 40 then
                      self.vibeDt = 0; 
--                      print("dt = "..dt);                 
--                      local node = ObjectEditor.indexToObject(ObjectEditor.mapNode, nearObjects);
                      xv, yv, zv = getRotation(self.nearestObject);
                      self.vibe = not self.vibe;
                      if self.vibe then
                          setRotation(self.nearestObject, xv+0.01, yv+0.01, zv+0.01);
                      else
                          setRotation(self.nearestObject, xv-0.01, yv-0.01, zv-0.01);
                      end
                  end
                  if self.cutTime > 2000 then
                      ObjectEditor.removeObject(self.nearestPath);
                      self.nearestObject = 0;
                  end
            end
            
        else
            self.hasNearestObject = false;
            self.cutTime = 0;        
        end
        self.player:lockInput(lockPlayerInput)
    end
end

function hedgeTrimmer:updateTick(dt, allowInput)
    if allowInput then
        local x,y,z = getWorldTranslation(self.rootNode)
--        local nearObjects = ObjectEditor.getNonCollisionObjects(x, z);
        if nearObjects ~= nil then
 --           local node = ObjectEditor.indexToObject(ObjectEditor.mapNode, nearObjects);
--            setMaterial(node, material, 0)
            
--            toggleVisibility(node);
        end
    end
end

function hedgeTrimmer:draw()
end

function hedgeTrimmer:setHandNode(handNode)
--    print("---setHandNode");
    HandTool.setHandNode(self, handNode);
    if self.currentHandNode ~= handNode then
        print(unpack(self.handNodeRotation))
        if g_currentMission.player ~= self.player then
            link(handNode, self.rootNode)
            self.currentHandNode = handNode
--            print(unpack(self.handNodeRotation))
            setRotation(self.rootNode, unpack(self.handNodeRotation))
            local x,y,z = getWorldTranslation(self.handNode)
            x,y,z = worldToLocal(getParent(self.rootNode), x,y,z)
            local a,b,c = getTranslation(self.rootNode)
            setTranslation(self.rootNode, a-x,b-y,c-z)
        end
    end
end

function hedgeTrimmer:onActivate(allowInput)
--    print("---Activate");
    HandTool.onActivate(self)
end

function hedgeTrimmer:onDeactivate(allowInput)
--    print("---Deactivate");
    HandTool.onDeactivate(self)
end

